use xmltest
go

-- update when changed
update statistics xmlinvoice
dbcc freeproccache

-- without an XML index

-- top-down - base table processed first (top of graphic showplan)
-- Base table - clustered index scan
-- TVF /Invoice (id, in output list)
-- TVF /Invoice/InvoiceID (id, value in output list)
select * from xmlinvoice 
where invoice.exist('/Invoice[@InvoiceID = "1003"]') = 1

-- query plan change? yes, better 
-- top-down
-- Base table - clustered index scan
-- TVF /Invoice/@InvoiceID (only looks for values in output list)
select * from xmlinvoice 
where invoice.exist('/Invoice/@InvoiceID[. = "1003"]') = 1

-- xml primary
create primary xml index invoiceidx ON xmlinvoice(invoice)

-- see columns in the node table
SELECT c.object_id, substring(c.name,1,10), c.column_id, c.system_type_id, c.user_type_id, max_length, precision FROM sys.columns c
 JOIN sys.indexes i ON i.object_id = c.object_id
 WHERE i.name = 'invoiceidx'
 AND i.type = 1
go

dbcc freeproccache

-- best choice of query from before index
-- bottom-up execution (XQuery done first, then joined with base table)
-- clustered index scan for InvoiceID, then filter on value
-- clustered index seek on base table
select * from xmlinvoice 
where invoice.exist('/Invoice/@InvoiceID[. = "1003"]') = 1

-- worst choice of query from before index
-- bottom-up execution (XQuery done first, then joined with base table)
-- clustered index scan for InvoiceID, then filter on value
-- clustered index seek on Invoice, then nested loop join
-- clustered index seek on base table
select * from xmlinvoice 
where invoice.exist('/Invoice[@InvoiceID = "1003"]') = 1

-- create path index (only)
create xml index invpathidx on xmlinvoice(invoice)
 using xml index invoiceidx for path
dbcc freeproccache

-- bottom-up execution (XQuery done first, then joined with base table)
-- index seek on the PATH index (for the value 1003)
-- clustered index seek on base table
select * from xmlinvoice 
where invoice.exist('/Invoice/@InvoiceID[. = "1003"]') = 1

-- now create the other two secondary indexes
create xml index invvalidx on xmlinvoice(invoice)
 using xml index invoiceidx for value

create xml index invpropidx on xmlinvoice(invoice)
 using xml index invoiceidx for property

dbcc freeproccache

-- now it uses the value index instead
-- bottom-up execution (XQuery done first, then joined with base table)
-- index seek on the VALUE index (for the value 1003)
-- clustered index seek on base table
select * from xmlinvoice 
where invoice.exist('/Invoice/@InvoiceID[. = "1003"]') = 1

-- because neither path nor value is selective enough
-- top-down execution
-- clustered index scan on base table
-- index seek on the PROPERTY index (for the text property)
select *
from xmlinvoice
where invoice.exist('/Invoice//CustomerName[text() = "Mary Weaver"]') = 1